/******************************************************************************
 * program:     rasimg library                                                *
 * function:    Executable for image conversion.                              *
 * modul:       convert.cc                                                    *
 * licency:     GPL or LGPL                                                   *
 ******************************************************************************/
#include <stddef.h>
#include <stdio.h>

#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "typedfs.h"
#include "raster.h"
#include "imageop.h"

#ifndef __UNIX__
 //#include <io.h>
#endif

#include <matrix.h>


//#include <alloc.h>

#ifndef __Have_stricmp
 #ifdef __Have_strcasecmp
  #define stricmp strcasecmp
  #define strnicmp strncasecmp
 #endif
 #ifndef stricmp        //already fixed?
  #define stricmp strcmp
 #endif
#endif


const char *GetExtension(const char *FullName)
{
const char *NameStep;
int LastPos;

 if(FullName==NULL) return(NULL);

 LastPos=strlen(FullName);
 NameStep=FullName+LastPos+1;
 while(NameStep>FullName)
 {
   NameStep--;

   if(*NameStep=='/' || *NameStep=='\\' || *NameStep==':')
		break;
   if(*NameStep=='.')
		return(NameStep);
 }

 return(FullName+LastPos);
}


/////////////////////Main CODE starts here/////////////////////////////////

char Verbosity = 0;

void PrintHeader(void)
{
char HdrPrinted=0;
  if(Verbosity>=0 && HdrPrinted==0)
  {
    printf("\n\r <<< CONVERT >>> Convert picture format (c)1997-2022 F&TSoft, version %u.%u\n", (RAS_IMG_VERSION>>8), (RAS_IMG_VERSION & 0xFF));
    HdrPrinted=1;
  }
}


void PrintHelp(void)
{
TImageFileHandler *LoadPicture_XXX;

  printf("Supported file formats are:\n");
  for(LoadPicture_XXX=TImageFileHandler::first();LoadPicture_XXX!=NULL;LoadPicture_XXX=LoadPicture_XXX->next())
  {
    printf("%s%s%s %s%s; ",LoadPicture_XXX->extension(),
			LoadPicture_XXX->description()==NULL?"":"#",
			LoadPicture_XXX->description()==NULL?"":LoadPicture_XXX->description(),
			LoadPicture_XXX->LoadPicture==NULL?"":"R",
			LoadPicture_XXX->SavePicture==NULL?"":"W");
  }
  printf(//"\n/NoScale      - means truncating image data"
	       "\n/Gray         - discards all color informations"
	       "\n/Separate[R|G|B] - remove one channel from true color images"
	       "\n/Format \"str\" - specify output format"
	       "\n/PeelPlane n  - cut only plane with number n"
	       "\n/PeelFrame n  - cut only scene with number n"
	       "\n/s            - decrease verbosity level"
	       "\n/v            - increase verbosity level"
	       "\n");
}


int main(int argc,char *argv[])
{
Image I1, I2;
const char *InputImg=NULL, *OutputImg=NULL;
const char *s;
const char *OutFormat=NULL;
int i;
int PeelPlaneNo = -1;
int PeelFrameNo = -1;
int Separate = -1;
int Error = 0;
bool Conv2Gray = false;
Image *CurrImg = &I1;


	/* Parse Arguments */
 for(i=1; i<argc; i++)
 {
   s = argv[i];
   if(s==NULL) continue;
   if(*s==0) continue;

   if(*s=='-' || *s=='/')
   {
     if(strcmp(s+1,"help")==0 || strcmp(s+1,"-help")==0 || strcmp(s+1,"?")==0)
     {
       PrintHeader();
       PrintHelp();
       return(0);
     }
     if(strcmp(s+1,"Format")==0 || strcmp(s+1,"format")==0)
     {
       ++i;
       if(i>=argc) break;
       OutFormat = argv[i];
       continue;
     }
     if(strcmp(s+1,"PeelFrame") == 0)
     {
       ++i;
       if(i>=argc) break;
       PeelFrameNo = atoi(argv[i]);
       if(PeelFrameNo <= 0)
       {
         printf("\n\rInvalid scene number: %d.", PeelFrameNo);
         goto End;
       }
       continue;
     }
     if(strcmp(s+1,"PeelPlane") == 0)
     {
       ++i;
       if(i>=argc) break;
       PeelPlaneNo = atoi(argv[i]);
       if(PeelPlaneNo <= 0)
       {
         printf("\n\rInvalid plane number: %d.", PeelPlaneNo);
         goto End;
       }
       continue;
     }
     if(strcmp(s+1,"s") == 0)
     {
       Verbosity = -1;
       continue;
     }
     if(strcmp(s+1,"v") == 0)
     {
       Verbosity = +1;
       continue;
     }
     if(strcmp(s+1,"Gray") == 0)
     {
       Conv2Gray = true;
       continue;
     }
     if(strcmp(s+1,"SeparateR") == 0)
     {
       Separate = 0;
       continue;
     }
     if(strcmp(s+1,"SeparateG") == 0)
     {
       Separate = 1;
       continue;
     }
     if(strcmp(s+1,"SeparateB") == 0)
     {
       Separate = 2;
       continue;
     }
     if(strcmp(s+1,"SeparateIDX") == 0)
     {
       Separate = 10;
       continue;
     }
     if(strcmp(s+1,"Separate") == 0)
     {
       ++i;
       if(i>=argc) break;
       if(strcmp(s,"R")==0) Separate=0;
       else if(strcmp(s,"G")==0) Separate=1;
       else if(strcmp(s,"B")==0) Separate=2;
       else if(strcmp(s,"IDX")==0) Separate=10;
       else
       {
         printf("\n\rInvalid channel for separiation: %s.", s);
         goto End;
       }
       continue;
     }
   }

  if(InputImg==NULL)
     InputImg=s;
  else
     if(OutputImg==NULL) OutputImg=s;
  }
	/* End of reading arguments */

  PrintHeader();
  if(Verbosity>=0) printf("\rLoading");
  //MarkTime=GetInterval();

//for(i=1;i<=10;i++)
  I1 = LoadPicture(InputImg);

  if(I1.Raster==NULL && I1.VecImage==NULL)
  {
    printf("\n\rCannot load input picture:\"%s\"",InputImg);
    goto End;
  }

  if(Verbosity>=0) printf("\rSaving ");
  //MarkTime=GetInterval();

  if(PeelFrameNo > 0)
  {
    while(PeelFrameNo>0 && CurrImg!=NULL)
    {
      if(PeelFrameNo==1) break;

      PeelFrameNo--;
      CurrImg->AttachRaster(NULL);
      CurrImg->AttachPalette(NULL);
      CurrImg = CurrImg->Next;
    }
    if(CurrImg==NULL)
    {
      printf("\n\rCannot obtain requested scene!");
      goto End;
    }
    if(CurrImg->Next != NULL)
    {
      delete CurrImg->Next;
      CurrImg->Next = NULL;
    }
  }

  if(Conv2Gray)
  {
    if(Verbosity>=0) printf("\rConverting to gray ");
    Convert2Gray(*CurrImg);
    CurrImg->AttachPalette(NULL);
  }
  
  if(Separate>=0)
  {
    if(Separate!=10)
      *CurrImg = Peel8bit(*CurrImg, Separate);	// R,G,B must be separated BEFORE palatte is cut.

    CurrImg->AttachPalette(NULL);
  }

  if(PeelPlaneNo>0)
  {    
    *CurrImg = PeelPlane(*CurrImg, PeelPlaneNo);
    CurrImg->AttachPalette(NULL);
  }

  if(OutFormat==NULL)
    Error = SavePicture(OutputImg,*CurrImg);  //No format requested
  else
    {
    TImageFileHandler *SavePicture_XXX;
    const char *ext;

    Error = 0xFFFF;
  
    for(SavePicture_XXX=TImageFileHandler::first();SavePicture_XXX!=NULL;SavePicture_XXX=SavePicture_XXX->next())
      {
      if(SavePicture_XXX->SavePicture!=NULL)
        {
        ext = SavePicture_XXX->extension();

        if(ext!=NULL)
          {
          if(*ext=='.') ext++;
	  i = strlen(ext);
          if(!stricmp(OutFormat,ext))
            {
	    Error = SavePicture_XXX->SavePicture(OutputImg,*CurrImg);	  //Exact match
            break;
	    }
	  if(!strnicmp(OutFormat,ext,i))
            {
	    if(OutFormat[i]=='#')
	      {
	      if(!stricmp(OutFormat+i+1,SavePicture_XXX->description()))
		{
		Error = SavePicture_XXX->SavePicture(OutputImg,*CurrImg);  //Description match
		break;
		}	   
	      }
	    }
          }
        }
      }
    }

  if(Error)
  {
    if(OutFormat && Error==0xFFFF)
      printf("\n\rNo export format:\"%s\" found.",OutFormat);
    else
      printf("\n\rCannot save output picture:\"%s\" Error %d",OutputImg,Error);
  }
  else
  {
    if(Verbosity>=0)
    {
      //printf("time : %f",abs(GetInterval()-MarkTime)/18.2);
      puts("\rOK     ");
    }
  }

End:
  return(Error);
}

